use SQLSatMetadataDemo

--------------------------------------------------------------------------------------
-- Integration Metadata Table Create Script
--
-- SQL Saturday, 1/30/2010, Richmond Va Presentation
-- NORMALIZE METADATA FOR DATA INTEGRATION ANALYSIS
-- Bob Lambert, mailto:Bob@RobertLambert.net, http://robertlambert.net
--
--------------------------------------------------------------------------------------
	
-- This script first drops objects created later in the script, in order to make this script rerunnable
-- During development make ddl changes to the dbo schema here
-- *the stage schema is created in the CreateInterfaceStageInbound script
-- *modify this script per your installation's standards, especially this section.
-- ****this section will delete all analysis metadata tables****

if exists(Select 1 from information_schema.tables where table_name = 'ReqmtMapping')
    drop table ReqmtMapping
	go
if exists(Select 1 from information_schema.tables where table_name = 'InterfaceMapping')
    drop table InterfaceMapping
	go
		
if exists(Select 1 from information_schema.tables where table_name = 'DataReqmt')
    drop table DataReqmt
	go
	
if exists(Select 1 from information_schema.tables where table_name = 'Requirement')
    drop table Requirement
	go

if exists(Select 1 from information_schema.tables where table_name = 'DBColumn')
    drop table DBColumn
	go	
if exists(Select 1 from information_schema.tables where table_name = 'DBTable')
    drop table DBTable
	go	
if exists(Select 1 from information_schema.tables where table_name = 'SubjectArea')
    drop table SubjectArea
	go	

if exists(Select 1 from information_schema.tables where table_name = 'Field')
    drop table Field
	go	
if exists(Select 1 from information_schema.tables where table_name = 'Interface')
    drop table Interface
	go	

-- Create Analysis Metadata Tables

Create table Requirement (
	RequirementId int identity (1,1) not null,
	RequirementLabel nvarchar(255),
	RequirementText nvarchar(255),
	RequirementNotesText nvarchar(255),
	VersionNumber tinyint,
	ReleaseNumber tinyint,
	PriorityNumber tinyint,
	LoadDateTime datetime NOT NULL ,
	ModDateTime datetime NULL,
	CreatedByUserName nvarchar(255),
	ModifiedByUserName nvarchar(255),
	CONSTRAINT PK_Requirement PRIMARY KEY  
		(RequirementId asc) 
	) 
	go

Create table DataReqmt (
	RequirementId int NOT NULL,
	DataReqmtId int identity (1,1) not null,
	DataReqmtName nvarchar(255),
	DataReqmtCategoryText nvarchar(255),
	DataReqmtDefinitionText nvarchar(255), 
	EffectiveDate date,
	EndDate date,
	LoadDateTime datetime NOT NULL ,
	ModDateTime datetime NULL,
	CreatedByUserName nvarchar(255),
	ModifiedByUserName nvarchar(255),
	CONSTRAINT PK_DataReqmt PRIMARY KEY (DataReqmtId asc), 
	CONSTRAINT FK_Requriement_DataReqmt Foreign Key (RequirementId) 
		references Requirement (RequirementId)
	) 
	go


CREATE TABLE dbo.SubjectArea(
	SubjectAreaId int IDENTITY(1,1) NOT NULL,
	SubjectAreaName nvarchar(255) NULL,
	LoadDateTime datetime NOT NULL ,
	ModDateTime datetime NULL,
	CreatedByUserName nvarchar(255),
	ModifiedByUserName nvarchar(255),
	CONSTRAINT PK_SubjectArea PRIMARY KEY (SubjectAreaId ASC)
) 


CREATE TABLE dbo.DBTable(
	TableId int IDENTITY(1,1) NOT NULL,
	SubjectAreaId int Not NULL,
	TableName nvarchar(255) NULL,
	LoadDateTime datetime NOT NULL ,
	ModDateTime datetime NULL,
	CreatedByUserName nvarchar(255),
	ModifiedByUserName nvarchar(255),
	CONSTRAINT PK_DBTable PRIMARY KEY  
	(SubjectAreaID ASC, TableId ASC),
	CONSTRAINT FK_SubjectArea_Table Foreign Key (SubjectAreaId) 
		references SubjectArea (SubjectAreaId)
)


CREATE TABLE dbo.DBColumn(
	SubjectAreaId int Not NULL,
	TableId int NOT NULL,
	ColumnId int IDENTITY(1,1) NOT NULL,
	ColumnName nvarchar(255) NULL,
	ColumnDataTypeText nvarchar(255) NULL,
	ColumnDataLengthNumber smallint null,
	ColumnPrecisionNumber smallint null,
	ColumnDefinitionText nvarchar(255) NULL,
	LoadDateTime datetime NOT NULL ,
	ModDateTime datetime NULL,
	CreatedByUserName nvarchar(255),
	ModifiedByUserName nvarchar(255),
	CONSTRAINT PK_DBColumn PRIMARY KEY  
	(SubjectAreaID ASC, TableId asc, ColumnID asc),
	CONSTRAINT FK_Table_Column Foreign Key (SubjectAreaId, TableId) 
		references DBTable (SubjectAreaId, TableId)
 ) 


CREATE TABLE dbo.Interface(
	InterfaceId int IDENTITY(1,1) NOT NULL,
	InterfaceNameText nvarchar(255) NULL,
	DirectionNameText nvarchar(255) NULL,
	LoadDateTime datetime NOT NULL ,
	ModDateTime datetime NULL,
	CreatedByUserName nvarchar(255),
	ModifiedByUserName nvarchar(255),
	CONSTRAINT PK_Interface PRIMARY KEY (InterfaceId ASC)
 )


CREATE TABLE dbo.Field( 
	InterfaceId int NOT NULL,
	FieldId int IDENTITY(1,1) NOT NULL,
	FieldGroupNameText nvarchar(255),
	FieldNameText nvarchar(255) NULL,
	DatatypeText nvarchar(255) NULL,
	FieldLengthNum smallint NULL,
	FieldRequiredFlag bit NULL,
	ValidValuesText nvarchar(255) NULL,
	LoadDateTime datetime NOT NULL ,
	ModDateTime datetime NULL,
	CreatedByUserName nvarchar(255),
	ModifiedByUserName nvarchar(255),
	CONSTRAINT PK_InterfaceField 
		PRIMARY KEY (InterfaceId ASC, FieldId ASC),
	CONSTRAINT FK_Interface_Field Foreign Key (InterfaceId) 
		references Interface (InterfaceId)
)

Create Table ReqmtMapping (
	ReqmtMappingId Int Identity(1,1) not null,
	DataReqmtId Int not null,
	SubjectAreaId int Not NULL,
	TableId int NOT NULL,
	ColumnId int NULL,
	Condition nvarchar(255),
	LoadDateTime datetime NOT NULL ,
	ModDateTime datetime NULL,
	CreatedByUserName nvarchar(255),
	ModifiedByUserName nvarchar(255),
	CONSTRAINT PK_ReqmtMapping PRIMARY KEY CLUSTERED (ReqmtMappingId asc), 
	CONSTRAINT FK_ReqmtMapping_Column Foreign Key (SubjectAreaId, TableId, ColumnId) 
		references DBColumn (SubjectAreaId, TableId, ColumnId),
	CONSTRAINT FK_ReqmtMapping_DataReqmt Foreign Key (DataReqmtId) 
		references DataReqmt (DataReqmtId)
)

Create Table InterfaceMapping (
	InterfaceMappingId Int Identity(1,1) not null,
	InterfaceId Int not null,
	FieldId	Int not null,
	SubjectAreaId int Not NULL,
	TableId int NOT NULL,
	ColumnId int NOT NULL,
	Transformations nvarchar(255),
	Comments nvarchar(255),	
	LoadDateTime datetime NOT NULL ,
	ModDateTime datetime NULL,
	CreatedByUserName nvarchar(255),
	ModifiedByUserName nvarchar(255),
	CONSTRAINT PK_InterfaceMapping PRIMARY KEY CLUSTERED (InterfaceMappingId asc), 
	CONSTRAINT FK_InterfaceMapping_Column Foreign Key (SubjectAreaId, TableId, ColumnId) 
		references DBColumn (SubjectAreaId, TableId, ColumnId),
	CONSTRAINT FK_InterfaceMapping_Field Foreign Key (InterfaceId, FieldId) 
		references Field (InterfaceId, FieldId)

)

-- Load Subject Area table with default value for demo

INSERT INTO dbo.SubjectArea
		(SubjectAreaName
		,LoadDateTime
		,CreatedByUserName
        )
     VALUES
		('Internet Sales'
		,GETDATE()
		,User)
          
GO

-- Load DBTable and DBColumn tables with values from the SQL 2008 Information Schema
-- For demo using the AdventureWorksDW database

INSERT INTO dbo.DBTable
           (SubjectAreaId
           ,TableName
           ,LoadDateTime
           ,CreatedByUserName)
  select 
		 1
		,  TABLE_NAME
		, GETDATE()
		, USER
	from AdventureWorksDW.INFORMATION_SCHEMA.TABLES
GO


INSERT INTO dbo.DBColumn
           (SubjectAreaId
           ,TableId
           ,ColumnName
           ,ColumnDataTypeText
           ,ColumnDataLengthNumber
           ,ColumnPrecisionNumber
           ,LoadDateTime
           ,CreatedByUserName)
   	Select
		  DBTable.SubjectAreaId
		, DBTable.TableId
		, ISColumns.COLUMN_NAME
		, ISColumns.DATA_TYPE
		, isnull(ISColumns.CHARACTER_MAXIMUM_LENGTH,iscolumns.NUMERIC_PRECISION)
		, ISColumns.NUMERIC_SCALE
		, GETDATE()
		, USER
     from AdventureWorksDW.INFORMATION_SCHEMA.COLUMNS as ISColumns
     Inner join DBTable on (ISColumns.TABLE_NAME = DBTable.TableName)

GO

